indexing
	description: "A timestamp string of the form: YYYY-MM-DD hh:mm:ss"
	license: "Eiffel Forum License v2 - See license.txt"
	source: "{
		Amalasoft Corporation
		273 Harwood Avenue
		Littleton, MA 01460 USA
		}"
	status: "Copyright 1995-2008, Amalasoft"
	date: "$Date: 2008/04/26 $"
	revision: "$Revision: 002$"
	howto: "{
	Create an instance of the this class using any of the listed
	creation procedures.
	If you want to mark local time, use either make or make_fine_grained.
	If you want to mark UTC time, use either make_utc or make_fine_utc.
	Because this is a descendent of STRING, the string representation
	is simply Current and references to objects of this class can be
	treated as if they were strings.
	If you set the associated date bia set_from_date_time, be sure to
	update the string rendition every time the external data object is
	updated (if at all).  The string representation is kept up-to-date
	automatically in all other cases.
	This class depends on the Eiffel Base libraries, the TIME library
	from Eiffel Software, and the AEL_PRINTF library from Amalasoft.
	}"

class AEL_TIMESTAMP

inherit
	STRING
		rename
			make as str_make
		end

create
	make, make_utc, make_fine_grained, make_fine_utc

--|========================================================================
feature {NONE} -- Creation
--|========================================================================

	make is
			-- Create a new timestamp object and initialize it to the current 
			-- local time
		do
			str_make (24)
			set_to_present
		end

--|--------------------------------------------------------------

	make_utc is
			-- Create a new timestamp object and initialize it to the current 
			-- UTC time (GMT)
		do
			is_utc := True
			make
		end

--|--------------------------------------------------------------

	make_fine_grained is
			-- Create a new fine-grained timestamp object and initialize it to
			-- the current local time
		do
			is_fine_grained := True
			make
		end

--|--------------------------------------------------------------

	make_fine_utc is
			-- Create a new fine-grained timestamp object and initialize it to
			-- the current UTC time (GMT)
		do
			is_utc := True
			make_fine_grained
		end

--|========================================================================
feature -- Status
--|========================================================================

	is_fine_grained: BOOLEAN
			-- Is this timestamp fine-grained (i.e. does it show partial seconds)?

	is_utc: BOOLEAN
			-- Is this timestamp based on UTC?

	date: DATE_TIME
			-- Date-time object currently associated with this timestamp

--|========================================================================
feature -- Status Setting
--|========================================================================

	set_to_present is
			-- Reset this timestamp to the current time (either local of UTC)
		do
			if is_utc then
				create date.make_now_utc
			else
				create date.make_now
			end
			generate
		end

--|--------------------------------------------------------------

	set_from_date_time (v: DATE_TIME) is
			-- Reset this timestamp to the time represented by the given argument
		require
			arg_exists: v /= Void
		do
			date := v
			generate
		ensure
			has_date_by_reference: date = v
		end

--|========================================================================
feature {NONE} -- Implementation
--|========================================================================

	generate is
			-- Generate the string representation of the time held by the
			-- associated date-time object
		local
			pf: AEL_PRINTF
			ts: STRING
		do
			create pf
			if is_fine_grained then
				ts := pf.aprintf (timestamp_format_string,
				                  << date.year, date.month, date.day,
										date.hour, date.minute, date.fine_second >>)
			else
				ts := pf.aprintf (timestamp_format_string,
				                  << date.year, date.month, date.day,
										date.hour, date.minute, date.second >>)
			end
			clear_all
			append (ts)
		ensure
			generated: not is_empty
		end

	--|--------------------------------------------------------------

	timestamp_format_string: STRING is
			-- Formt string to use when generating the string representation
			-- of this timestamp
		do
			if is_fine_grained then
				Result := "%%04d-%%02d-%%02d %%02d:%%02d:%%06.3f%N"
			else
				Result := "%%04d-%%02d-%%02d %%02d:%%02d:%%02d"
			end
		ensure
			exists: Result /= Void
		end

	--|--------------------------------------------------------------

	terse_timestamp_format_string: STRING is
			-- Formt string to use when generating the terse
			-- string representation of this timestamp
		do
			if is_fine_grained then
				Result := "%%04d%%02d%%02d%%02d%%02d%%06.3f%N"
			else
				Result := "%%04d%%02d%%02d%%02d%%02d%%02d"
			end
		ensure
			exists: Result /= Void
		end

--|========================================================================
feature -- Special terse timestamp
--|========================================================================

	generate_terse is
			-- Generate the terse string representation of the time held by the
			-- associated date-time object
		local
			pf: AEL_PRINTF
			ts: STRING
		do
			create pf
			if is_fine_grained then
				ts := pf.aprintf (terse_timestamp_format_string,
				                  << date.year, date.month, date.day,
										date.hour, date.minute, date.fine_second >>)
			else
				ts := pf.aprintf (terse_timestamp_format_string,
				                  << date.year, date.month, date.day,
										date.hour, date.minute, date.second >>)
			end
			clear_all
			append (ts)
		ensure
			generated: not is_empty
		end

--|--------------------------------------------------------------
invariant
	has_date: date /= Void
	has_form: not is_empty

end -- class AEL_TIMESTAMP
